#include <gint/timer.h>
#include <gint/clock.h>
#include <gint/keyboard.h>

#include "core.h"
#include "mutation_engine.h"
#include "display_engine.h"


void next_frame(struct game *current_game)
{
    for (int i = 0; current_game->planes[i]; i++)
    {
        switch(current_game->planes[i]->direction)
        {
            case 1:
                current_game->planes[i]->y -= 1;
                break;
            case 2:
                current_game->planes[i]->x += 1;
                break;
            case 3:
                current_game->planes[i]->y += 1;
                break;
            case 4:
                current_game->planes[i]->x -= 1;
                break;
        }
        if (current_game->planes[i]->x == current_game->planes[i]->dest_x && current_game->planes[i]->y == current_game->planes[i]->dest_y)
        {
            // Set the new destination
            current_game->planes[i]->dest_x = current_game->planes[i]->depa_x;
            current_game->planes[i]->dest_y = current_game->planes[i]->depa_y;

            // Set the new departure
            current_game->planes[i]->depa_x = current_game->planes[i]->x;
            current_game->planes[i]->depa_y = current_game->planes[i]->y;

            // Set the new direction
            int new_dir = (current_game->planes[i]->direction + 2) % 4;
            if (!new_dir) new_dir = 4;
            current_game->planes[i]->direction = new_dir;
        }
    }
    current_game->time += ENGINE_TICK;
}


int get_inputs(const int background, int *mutation_menu)
{
    int key = rtc_key();

    if (key == KEY_OPTN && (background == 1 || background == 2)) return (background % 2) + 1;
    if (key == KEY_VARS) return 3;
    if (key == KEY_SQUARE) return 6;

    if (key == KEY_EXIT)
    {
        if (background == 5) return 3;
        if (background != 1 && background != 2) return 1;
        if (background == 1 || background == 2) return -1;
    }

    if (background == 3)
    {
        switch (key)
        {
            // Symptoms
            case KEY_F1:
                *mutation_menu = 1;
                return 5;
                break;

            // Abilities
            case KEY_F3:
                *mutation_menu = 2;
                return 5;
                break;

            // Transmission
            case KEY_F5:
                *mutation_menu = 3;
                return 5;
                break;

            // Return to the main menu
            case KEY_F6:
                *mutation_menu = 0;
                return 1;
                break;
        }
    }

    return background;
}


int rtc_key(void)
{
    int opt = GETKEY_DEFAULT & ~GETKEY_REP_ARROWS;
    int timeout = 1;
        
    key_event_t ev = getkey_opt(opt, &timeout);
    if(ev.type == KEYEV_NONE) return 0;
        
    return ev.key;
}


int callback_tick(volatile int *tick)
{
    *tick = 1;
    return TIMER_CONTINUE;
}


void manage_mutation(struct game *current_game, const int mutation_menu)
{
    int key = 0, end = 0;
    struct cursor c = {0, 0, 0};

    static volatile int tick = 1;
    int t = timer_configure(TIMER_ANY, CURSOR_TICK*1000, GINT_CALL(callback_tick, &tick));
    if (t >= 0) timer_start(t);


    while (!end)
    {   
        // Cursor blinking gestion
        while (!tick) sleep();
        tick = 0;
        c.display = (c.display + 1) % 2;

        // Get and display the mutation menu
        int table[4][8];
        get_mutation(current_game, mutation_menu, table);
        display_mutation(table, c, mutation_menu);
        
        // Get the key
        key = rtc_key();
        
        // Manage input
        if (key == KEY_EXIT) end = 1;
        if (key == KEY_EXE && table[c.y][c.x] != 15 && table[c.y][c.x] != 0) mutation_buy(current_game, c, mutation_menu, table);
                    
        if (key == KEY_LEFT && c.x > 0) c.x--;
        if (key == KEY_RIGHT && c.x < 7) c.x++;
        if (key == KEY_UP && c.y > 0) c.y--;
        if (key == KEY_DOWN && c.y < 3) c.y++;
    }
    if (t >= 0) timer_stop(t);
}


void mutation_buy(struct game *current_game, const struct cursor c, const int mutation_menu, const int table[4][8])
{
    int button_selected = 1, end = 0;
    int key = 0;

    while (!end)
    {
        mutation_selected(c, mutation_menu, table, button_selected);
        key = getkey().key;

        if (key == KEY_DOWN || key == KEY_UP) button_selected = (button_selected + 1) % 2;
        if (key == KEY_EXIT) end = 1;
        if (key == KEY_EXE)
        {
            if (!button_selected) end = 1;
            // else evolution
        }

    }
}
